﻿local addonName, addon = ...

local L, settings = addon.L

local ICON = [[Interface\AddOns\]] .. addonName .. [[\Icon]]

--[[----------------------------------------------------------------------------
Default settings
------------------------------------------------------------------------------]]
local defaultSettings = {
	closeWithEsc				= true,

	-- Position
	width						= 384,
	height						= 260,
	point						= 'CENTER',
	relFrame					= nil,
	relPoint					= 'CENTER',
	offsetX						= 0,
	offsetY						= 0,

	-- cvars
	flashErrorMessageRepeats	= true,
	scriptErrors				= true,
	scriptWarnings				= true,

	-- Events
	ADDON_ACTION_BLOCKED		= true,
	ADDON_ACTION_FORBIDDEN		= true,
	LUA_WARNING					= true,
	MACRO_ACTION_BLOCKED		= true,
	MACRO_ACTION_FORBIDDEN		= true,
	SAVED_VARIABLES_TOO_LARGE	= true,
	SYSMSG						= true,
	UI_ERROR_MESSAGE			= true,
	UI_INFO_MESSAGE				= true
}

addon.cvars = {
	flashErrorMessageRepeats	= 'UIErrorsFrame',
	scriptErrors				= 'ScriptErrorsFrame',
	scriptWarnings				= 'ScriptErrorsFrame'
}

addon.events = {
	ADDON_ACTION_BLOCKED		= 'UIParent',
	ADDON_ACTION_FORBIDDEN		= 'UIParent',
	LUA_WARNING					= 'UIParent',
	MACRO_ACTION_BLOCKED		= 'UIParent',
	MACRO_ACTION_FORBIDDEN		= 'UIParent',
	SAVED_VARIABLES_TOO_LARGE	= 'UIParent',
	SYSMSG						= 'UIErrorsFrame',
	UI_ERROR_MESSAGE			= 'UIErrorsFrame',
	UI_INFO_MESSAGE				= 'UIErrorsFrame'
}

--[[----------------------------------------------------------------------------
API
------------------------------------------------------------------------------]]
function addon.ApplySettings()
	local UISpecialFrames = UISpecialFrames
	for index = #UISpecialFrames, 1, -1 do
		if UISpecialFrames[index] == 'ScriptErrorsFrame' then
			tremove(UISpecialFrames, index)
		end
	end
	if settings.closeWithEsc then
		UISpecialFrames[#UISpecialFrames + 1] = 'ScriptErrorsFrame'
	end
end

function addon.InitializeSettings()
	for cvar, frameName in pairs(addon.cvars) do
		SetCVar(cvar, settings[cvar] and '1' or '0')
	end
	for event, frameName in pairs(addon.events) do
		_G[frameName][settings[event] and 'RegisterEvent' or 'UnregisterEvent'](_G[frameName], event)
	end
	addon.ApplySettings()
end

function addon.SaveScriptErrorsFramePosition()
	ScriptErrorsFrame:SetUserPlaced(false)
	local width, height = ScriptErrorsFrame:GetSize()
	local point, relFrame, relPoint, offsetX, offsetY = ScriptErrorsFrame:GetPoint(1)
	if type(relFrame) == 'table' then
		local name = relFrame:GetName()
		if type(name) == 'string' then
			relFrame = name
		else
			point, relFrame, relPoint, offsetX, offsetY = nil, nil, nil, nil, nil
		end
	end
	settings.width, settings.height = width, height
	settings.point, settings.relFrame, settings.relPoint, settings.offsetX, settings.offsetY = point, relFrame, relPoint, offsetX, offsetY
end

function addon.SetScriptErrorsFramePosition()
	ScriptErrorsFrame:ClearAllPoints()
	ScriptErrorsFrame:SetSize(settings.width, settings.height)
	ScriptErrorsFrame:SetPoint(settings.point, settings.relFrame, settings.relPoint, settings.offsetX, settings.offsetY)
end

--[[----------------------------------------------------------------------------
Slash command
------------------------------------------------------------------------------]]
local function ChangeErrorIndex(index)
	local numErrors = #ScriptErrorsFrame.order
	if not index then index = numErrors end
	if index < 0 then 
		index = numErrors + index + 1
	end
	if index >= numErrors then													-- Use >= to avoid else with 0
		index = numErrors
	elseif index < 1 then
		index = 1
	end
	if index ~= ScriptErrorsFrame.index then
		ScriptErrorsFrame.index = index
		ScriptErrorsFrame:Update()
	end
end

_G['SLASH_' .. addonName .. 1] = "/err"
_G['SLASH_' .. addonName .. 2] = "/error"
_G['SLASH_' .. addonName .. 3] = "/errors"
SlashCmdList[addonName] = function(index)
	if index ~= "" then
		ChangeErrorIndex(tonumber(index))
		ScriptErrorsFrame:Show()
	else
		ScriptErrorsFrame:SetShown(not ScriptErrorsFrame:IsShown())
	end
end

--[[----------------------------------------------------------------------------
LoD support
------------------------------------------------------------------------------]]
local LOA = LibStub('LibOptionsAssist-1.1', true)
if LOA and select(2, GetAddOnInfo(addonName .. '_Config')) then
	addon.config = LOA.AddEntry(addonName, nil, { addonName .. "_Config", addonName, addon, L })
end

--[[----------------------------------------------------------------------------
Plugin
------------------------------------------------------------------------------]]
local LDB = LibStub('LibDataBroker-1.1', true)
if LDB then
	local tooltip

	local function UpdateTooltip()
		local red, green, blue = NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b
		tooltip:ClearLines()
		tooltip:SetText(addonName, 1, 1, 1)
		if ScriptErrorsFrame then
			tooltip:AddDoubleLine(L["Left-Click"], L["Toggle Error Window"], 0.15, 0.85, 0.15, red, green, blue, true)
			if ScriptErrorsFrame:IsVisible() then
				tooltip:AddDoubleLine(L["Mouse Wheel"], L["Change Error Shown"], 0.15, 0.85, 0.15, red, green, blue, true)
			end
		end
		if addon.config then
			tooltip:AddDoubleLine(L["Right-Click"], L["Toggle Configuration"], 0.15, 0.85, 0.15, red, green, blue, true)
		end
		tooltip:Show()
	end

	addon.plugin = LDB:NewDataObject(addonName, {
		type = 'data source',
		icon = ICON,
		iconCoords = { 0.07, 0.93, 0.17, 0.93 },
		label = addonName,

		OnClick = function(_, button)
			if button == 'LeftButton' then
				ScriptErrorsFrame:SetShown(not ScriptErrorsFrame:IsShown())
			elseif button == 'RightButton' and addon.config then
				addon.config(true)
			end
		end,

		OnEnter = function(self)
			tooltip = GameTooltip
			tooltip:SetOwner(self, 'ANCHOR_NONE')

			if self.AttachFrame then
				self:AttachFrame(tooltip)
			else
				tooltip:ClearAllPoints()
				local _, pluginCenter = self:GetCenter()
				local _, uiCenter = UIParent:GetCenter()
				if pluginCenter >= uiCenter then
					tooltip:SetPoint('TOP', self, 'BOTTOM', 0, 0)
				else
					tooltip:SetPoint('BOTTOM', self, 'TOP', 0, 0)
				end
			end

			UpdateTooltip()
		end,

		OnLeave = function(self)
			if tooltip then
				if tooltip:IsOwned(self) then
					tooltip:Hide()
				end
				tooltip = nil
			end
		end,

		OnMouseWheel = function(_, delta)
			if ScriptErrorsFrame and ScriptErrorsFrame:IsVisible() then
				ChangeErrorIndex((ScriptErrorsFrame.index or 0) + delta)
			end
		end
	})

	local function LDB_OnHideShowHook()
		if tooltip then
			UpdateTooltip()
		end
	end
	ScriptErrorsFrame:HookScript('OnHide', LDB_OnHideShowHook)
	ScriptErrorsFrame:HookScript('OnShow', LDB_OnHideShowHook)

	hooksecurefunc(ScriptErrorsFrame, 'UpdateButtons', function()
		local numErrors = #ScriptErrorsFrame.order
		if numErrors > 0 then
			addon.plugin.icon = [[Interface\Icons\Trade_Archaeology_Insect in Amber]]
			addon.plugin.value = numErrors
		else
			addon.plugin.icon = ICON
			addon.plugin.value = nil
		end
	end)
end

--[[----------------------------------------------------------------------------
Initialize
------------------------------------------------------------------------------]]
local frame = CreateFrame('Frame')
frame:Hide()

local addonLoaded, varsLoaded
frame:SetScript('OnEvent', function(self, event, name)
	if event == 'ADDON_LOADED' then
		if name ~= addonName then return end

		settings = _G[addonName .. "Settings"]
		if type(settings) ~= 'table' then
			settings = { }
			_G[addonName .. "Settings"] = settings
		end
		setmetatable(settings, { __index = defaultSettings })
		addon.settings = settings

		addonLoaded = true
	elseif event == 'VARIABLES_LOADED' then
		varsLoaded = true
	end
	self:UnregisterEvent(event)
	if addonLoaded and varsLoaded then
		self:SetScript('OnEvent', addon.SaveScriptErrorsFramePosition)
		self:RegisterEvent('PLAYER_LOGOUT')
		addon.InitializeSettings()
		addon.SetScriptErrorsFramePosition()
		ScriptErrorsFrame:UpdateButtons()
		self:SetScript('OnUpdate', function(self)
			self:SetScript('OnUpdate', nil)
			self:Hide()
			ScriptErrorsFrame:SetShown(#ScriptErrorsFrame.order > 0 and GetCVarBool('scriptErrors'))
		end)
		self:Show()
	end
end)
frame:RegisterEvent('ADDON_LOADED')
frame:RegisterEvent('VARIABLES_LOADED')
